-- Delete all the non-unique name/producer_id combinations from query table
-- Preserves newest queries out of non-unique ones
CREATE OR REPLACE FUNCTION add_unique_query_index() RETURNS void AS 
$$
	DECLARE current_query_id INTEGER;
	DECLARE current_query_name TEXT;
	DECLARE new_query_name TEXT;
	DECLARE current_producer_id INTEGER;
	DECLARE current_producer_name TEXT;
	DECLARE current_portal_id INTEGER;
	DECLARE current_portal_name TEXT;
	
BEGIN
	FOR current_query_id IN SELECT DISTINCT query.id FROM misp2.query AS query INNER JOIN misp2.query AS query2 ON 
	   query.producer_id = query2.producer_id AND query.name = query2.name AND query.id < query2.id LOOP
	   
	   SELECT INTO current_query_name, current_producer_id, current_producer_name, current_portal_id, current_portal_name 
	     query.name, producer.id, producer.short_name, portal.id, portal.short_name FROM misp2.query AS query 
			INNER JOIN misp2.producer AS producer ON query.producer_id = producer.id 
			INNER JOIN misp2.portal AS portal ON producer.portal_id = portal.id 
			WHERE query.id = current_query_id;
		-- Derive new query name by adding query.id to the non-unique query name, turning it into unique entry
		new_query_name := current_query_name || '_' || current_query_id;
		-- Make sure new name is not too long to fit 50 char limit, if it doesn't fit, crop the name from beginning so that 50 char names would also be turned to unique ones
		new_query_name := SUBSTRING(new_query_name FROM GREATEST(1, char_length(new_query_name) - 49) FOR LEAST(50, char_length(new_query_name)));
		UPDATE misp2.query SET name = new_query_name WHERE query.id = current_query_id;
	    RAISE NOTICE 'Renaming non-unique query %(%) | producer %(%) | portal %(%) to %', 
			current_query_name, current_query_id, current_producer_name, current_producer_id, current_portal_name, current_portal_id, new_query_name;
	   
	END LOOP;
	-- Delete index if it exists for whatever reason
	DROP INDEX IF EXISTS misp2.in_query;
	-- Create unque index on query.name
	CREATE UNIQUE INDEX in_query ON misp2.query (producer_id, name);

END $$ LANGUAGE plpgsql VOLATILE;
SELECT add_unique_query_index();
DROP FUNCTION add_unique_query_index();
